@@ -360,6 +360,12 @@ class StandardFunction:
         time_where_data_vector_less_zero = time[(np.where(data_vector < 0))]
         return time_where_data_vector_less_zero[0] if len(time_where_data_vector_less_zero) != 0 else np.inf
 
+
+    @staticmethod
+    def extended_ccdf_weibull(x: float, a: float, b: float, c: float, d: float):
+
+        return 1.0 - np.exp(-np.exp(1 / b * np.log(d * x + c) - a / b))
+
     @staticmethod
     def ccdf_weibull(x: float, lambda_: float, k: float):
         r"""Cumulative distribution $f$ function for the Weibull distribution:
