@@ -3,7 +3,7 @@ import numpy as np
 from ..data.dynasaur_definitions import DynasaurDefinitions
 from ..utils.logger import ConsoleLogger
 from ..utils.constants import VPSDataConstant, DataChannelTypesNodout, DataChannelTypesSecforc, DataChannelTypesContact, \
-    DataChannelTypesGlobalEnergy
+    DataChannelTypesGlobalEnergy, DataChannelTypesRbdout, DataChannelTypesSbtout
 
 
 class VPSData:
@@ -43,6 +43,12 @@ class VPSData:
                 elif node.parent.name.split("/")[5] in [VPSDataConstant.SECTION]:
                     self._set_section_elements(node)
 
+                elif node.parent.name.split("/")[5] in [VPSDataConstant.JOINT]:
+                    self._set_joint_elements(node)
+
+                elif node.parent.name.split("/")[5] in [VPSDataConstant.SLIPRING]:
+                    self._set_sbtout_elements(node)
+
                 elif node.parent.name.split("/")[5] in [VPSDataConstant.CONTACT]:
                     self._set_contact_elements(node)
 
@@ -83,6 +89,13 @@ class VPSData:
 
                 elif args[0] == VPSDataConstant.MATSUM:
                     legend = f["/CSMEXPL/constant/attributes/PART/erfblock/title"][()]
+
+                elif args[0] == VPSDataConstant.RIGID_BODY:
+                    legend = f["/CSMEXPL/constant/attributes/JOINT/erfblock/title"][()]
+
+                elif args[0] == VPSDataConstant.SBTOUT:
+                    legend = f["/CSMEXPL/constant/attributes/SLIPRING/erfblock/title"][()]
+
                 else:
                     assert False
 
@@ -90,7 +103,7 @@ class VPSData:
                 leg = "".join([i.decode("utf-8").ljust(80) for i in legend])
                 return leg
 
-        elif args[1] == "ids":
+        elif args[1] == "ids" or args[0] == VPSDataConstant.SBTOUT and args[1] == "slipring_ids":
 
             # check if data has been cached
             if args[0] in self._ids:
@@ -114,6 +127,16 @@ class VPSData:
                     entid_translation = f["/CSMEXPL/constant/identifiers/NODE/erfblock/entid"][()]
                     uids = f["/CSMEXPL/constant/identifiers/NODE/erfblock/uid"][()]
 
+                elif args[0] == VPSDataConstant.RIGID_BODY:
+                    entids = f["/CSMEXPL/constant/attributes/JOINT/erfblock/entid"][()]
+                    entid_translation = f["/CSMEXPL/constant/identifiers/JOINT/erfblock/entid"][()]
+                    uids = f["/CSMEXPL/constant/identifiers/JOINT/erfblock/uid"][()]
+
+                elif args[0] == VPSDataConstant.SBTOUT:
+                    entids = f["/CSMEXPL/constant/attributes/"  + VPSDataConstant.SLIPRING + "/erfblock/entid"][()]
+                    entid_translation = f["/CSMEXPL/constant/identifiers/" + VPSDataConstant.SLIPRING + "/erfblock/entid"][()]
+                    uids = f["/CSMEXPL/constant/identifiers/"+ VPSDataConstant.SLIPRING + "/erfblock/uid"][()]
+
                 elif args[0] == VPSDataConstant.MATSUM:
                     entids = f["/CSMEXPL/constant/attributes/PART/erfblock/entid"][()]
                     entid_translation = f["/CSMEXPL/constant/identifiers/PART/erfblock/entid"][()]
@@ -140,7 +163,16 @@ class VPSData:
                 self._ids[args[0]] = ids
                 return ids
 
-        axis = 2
+        if args[0] == VPSDataConstant.GLSTAT:
+            data_type = VPSDataConstant.ENERGY_GLOBAL
+            idx, data_channel = self._get_global_energy_index_and_data_channel(args[1])
+            axis = 1
+
+            search_string = "/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + data_type + "/" + data_channel + "/ZONE1_set1/erfblock/res"
+
+            with h5py.File(self._vps_file_path, 'r') as f:
+                return np.linalg.norm(f[search_string][:, :, idx], axis=axis)
+
         if args[0] == VPSDataConstant.NODOUT:
             data_type = VPSDataConstant.NODE
             idx, data_channel = self._get_node_index_and_data_channel(args[1])
@@ -153,19 +185,30 @@ class VPSData:
             data_type = VPSDataConstant.CONTACT
             idx, data_channel = self._get_contact_index_and_data_channel(args[1])
 
-        elif args[0] == VPSDataConstant.GLSTAT:
-            data_type = VPSDataConstant.ENERGY_GLOBAL
-            idx, data_channel = self._get_global_energy_index_and_data_channel(args[1])
-            axis = 1
+        elif args[0] == VPSDataConstant.RIGID_BODY:
+            data_type = VPSDataConstant.JOINT
+            idx, data_channel = self._get_rbdout_index_and_data_channel(args[1])
+
+        elif args[0] == VPSDataConstant.SBTOUT:
+            data_type = VPSDataConstant.SLIPRING
+            idx, data_channel = self._get_sbtout_index_and_data_channel(args[1])
 
         elif args[0] == VPSDataConstant.MATSUM:
             data_type = VPSDataConstant.ENERGY_PART
             idx, data_channel = self._get_part_energy_index_and_data_channel(args[1])
 
+        else:
+            assert False
+
         search_string = "/CSMEXPL/multistate/TIMESERIES1/multientityresults/" + data_type + "/" + data_channel + "/ZONE1_set1/erfblock/res"
 
-        with h5py.File(self._vps_file_path, 'r') as f:
-            return np.linalg.norm(f[search_string][:, :, idx], axis=axis)
+        if isinstance(idx, list):
+            # list is returned, for resultant forces etc.
+            with h5py.File(self._vps_file_path, 'r') as f:
+                return np.linalg.norm(f[search_string][:, :, idx], axis=2)
+        else:
+            with h5py.File(self._vps_file_path, 'r') as f:
+                return f[search_string][:, :, idx]
 
     def read(self, *argv):
         """
@@ -217,37 +260,37 @@ class VPSData:
         # NODE
         if arg == DataChannelTypesNodout.X_COORDINATE or \
                 arg == DataChannelTypesNodout.Y_COORDINATE or arg == DataChannelTypesNodout.Z_COORDINATE:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.COORDINATE
 
         elif arg == DataChannelTypesNodout.X_DISPLACEMENT or \
                 arg == DataChannelTypesNodout.Y_DISPLACEMENT or arg == DataChannelTypesNodout.Z_DISPLACEMENT:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.TRANSLATION_DISPLACEMENT
 
         elif arg == DataChannelTypesNodout.X_VELOCITY or \
                 arg == DataChannelTypesNodout.Y_VELOCITY or arg == DataChannelTypesNodout.Z_VELOCITY:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.VELOCITY
 
         elif arg == DataChannelTypesNodout.X_ACCELERATION or \
                 arg == DataChannelTypesNodout.Y_ACCELERATION or arg == DataChannelTypesNodout.Z_ACCELERATION:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.ACCELERATION
 
         elif arg == DataChannelTypesNodout.RX_DISPLACEMENT or \
                 arg == DataChannelTypesNodout.RY_DISPLACEMENT or arg == DataChannelTypesNodout.RZ_DISPLACEMENT:
-            idx = [0] if arg.startswith("rx") else ([1] if arg.startswith("ry") else [2])
+            idx = 0 if arg.startswith("rx") else (1 if arg.startswith("ry") else 2)
             data_channel = VPSDataConstant.ROTATION_ANGLE
 
         elif arg == DataChannelTypesNodout.RX_VELOCITY or \
                 arg == DataChannelTypesNodout.RY_VELOCITY or arg == DataChannelTypesNodout.RZ_VELOCITY:
-            idx = [0] if arg.startswith("rx") else ([1] if arg.startswith("ry") else [2])
+            idx = 0 if arg.startswith("rx") else (1 if arg.startswith("ry") else 2)
             data_channel = VPSDataConstant.ROTATION_VELOCITY
 
         elif arg == DataChannelTypesNodout.RX_ACCELERATION or \
                 arg == DataChannelTypesNodout.RY_ACCELERATION or arg == DataChannelTypesNodout.RZ_ACCELERATION:
-            idx = [0] if arg.startswith("rx") else ([1] if arg.startswith("ry") else [2])
+            idx = 0 if arg.startswith("rx") else (1 if arg.startswith("ry") else 2)
             data_channel = VPSDataConstant.ROTATION_ACCELERATION
 
         else:
@@ -268,16 +311,16 @@ class VPSData:
         """
         # MATSUM
         if arg == "hourglass_energy":
-            idx = [0]
+            idx = 0
             data_channel = "MHGL"
         elif arg == "internal_energy":
-            idx = [0]
+            idx = 0
             data_channel = "MINT"
         elif arg == "kinetic_energy":
-            idx = [0]
+            idx = 0
             data_channel = "MKIN"
         elif arg == "added_mass":
-            idx = [0]
+            idx = 0
             data_channel = "DMSC"
         else:
             # TODO: add console logger
@@ -339,21 +382,23 @@ class VPSData:
         # SECTION
         if arg == DataChannelTypesSecforc.X_CENTROID or \
                 arg == DataChannelTypesSecforc.Y_CENTROID or arg == DataChannelTypesSecforc.Z_CENTROID:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.SECTION_CENTRE_POSITION
 
         elif arg == DataChannelTypesSecforc.X_FORCE or \
                 arg == DataChannelTypesSecforc.Y_FORCE or arg == DataChannelTypesSecforc.Z_FORCE:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.SECTION_FORCE
+
         elif arg == DataChannelTypesSecforc.TOTAL_FORCE:
             idx = [0, 1, 2]
             data_channel = VPSDataConstant.SECTION_FORCE
 
         elif arg == DataChannelTypesSecforc.X_MOMENT or \
                 arg == DataChannelTypesSecforc.Y_MOMENT or arg == DataChannelTypesSecforc.Z_MOMENT:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.SECTION_MOMENT
+
         elif arg == DataChannelTypesSecforc.TOTAL_MOMENT:
             idx = [0, 1, 2]
             data_channel = VPSDataConstant.SECTION_MOMENT
@@ -395,7 +440,7 @@ class VPSData:
         """
         if arg == DataChannelTypesContact.X_FORCE or \
                 arg == DataChannelTypesContact.Y_FORCE or arg == DataChannelTypesSecforc.Z_FORCE:
-            idx = [0] if arg.startswith("x") else ([1] if arg.startswith("y") else [2])
+            idx = 0 if arg.startswith("x") else (1 if arg.startswith("y") else 2)
             data_channel = VPSDataConstant.CONTACT_FORCE
 
         elif arg == DataChannelTypesSecforc.TOTAL_FORCE:
@@ -408,6 +453,48 @@ class VPSData:
 
         return idx, data_channel
 
+    def _get_sbtout_index_and_data_channel(self, arg):
+
+        if arg == DataChannelTypesSbtout.RING_SLIP:
+            idx = 0
+            data_channel = VPSDataConstant.SLIPRING_VARIABLES
+
+        else:
+            # TODO: add console logger
+            print("ID FAIL")
+            return
+
+        return idx, data_channel
+
+    def _get_rbdout_index_and_data_channel(self, arg):
+        """
+
+        :param arg:
+        :return:
+        """
+        print(arg)
+        # TODO
+        if arg == DataChannelTypesRbdout.LOCAL_RDZ:
+            idx = 0
+            data_channel = VPSDataConstant.JOINT_RELATIVE_ROTATION_R
+
+        # if arg == DataChannelTypesRbdout.LOCAL_RDY:
+        #     idx= 0
+        #     data_channel = VPSDataConstant.JOINT_RELATIVE_ROTATION_S_I
+        #
+        # if arg == DataChannelTypesRbdout.LOCAL_RDZ:
+        #     idx = 0
+        #     data_channel = VPSDataConstant.JOINT_RELATIVE_ROTATION_T_II
+
+
+        else:
+            # TODO: add console logger
+            print(f"{arg} for ID FAIL")
+            return
+
+        return idx, data_channel
+
+
     def _set_node_elements(self, node):
         """
 
@@ -529,6 +616,50 @@ class VPSData:
 
         # idx_of_data = node.parent.name.split("/").index(node.parent.name.split("/")[5]) + 1
 
+
+    def _set_sbtout_elements(self, node):
+        """
+
+        Args:
+          node:
+
+        Returns:
+
+        """
+        if node.parent.name.split("/")[5] == VPSDataConstant.SLIPRING:
+            if VPSDataConstant.SBTOUT not in self._d.keys():
+                self._d[VPSDataConstant.SBTOUT] = ["slipring_ids", "time", "legend"]
+            idx_of_data = node.parent.name.split("/").index(node.parent.name.split("/")[5]) + 1
+            #         # TODO
+            #         print("*******************************************")
+            #         print(node.parent.name.split("/")[idx_of_data])
+            #         print("-------------------------------------------")
+
+            if node.parent.name.split("/")[idx_of_data] == VPSDataConstant.SLIPRING_VARIABLES:
+                self._d[VPSDataConstant.SBTOUT].extend([DataChannelTypesSbtout.RING_SLIP])
+
+    def _set_joint_elements(self, node):
+        """
+
+        Args:
+          node:
+
+        Returns:
+
+        """
+        if node.parent.name.split("/")[5] == VPSDataConstant.JOINT:
+            if VPSDataConstant.RIGID_BODY not in self._d.keys():
+                self._d[VPSDataConstant.RIGID_BODY] = ["ids", "time", "legend"]
+            idx_of_data = node.parent.name.split("/").index(node.parent.name.split("/")[5]) + 1
+            #         # TODO
+            #         print("*******************************************")
+            #         print(node.parent.name.split("/")[idx_of_data])
+            #         print("-------------------------------------------")
+
+            if node.parent.name.split("/")[idx_of_data] == VPSDataConstant.JOINT_RELATIVE_ROTATION_R:
+                self._d[VPSDataConstant.RIGID_BODY].extend([DataChannelTypesRbdout.LOCAL_RDZ])
+
+
     def _set_section_elements(self, node):
         """
 
