@@ -174,8 +174,13 @@ class Plugin(object):
 
         elif JsonConstants.VALUE in json_object.keys():
             # return np.array(json_object[JsonConstants.VALUE]) * json_object[JsonConstants.UNIT]
+
+            unit = "dimensionless"
+            if JsonConstants.UNIT in json_object:
+                unit = json_object[JsonConstants.UNIT]
+
             return np.array(json_object[JsonConstants.VALUE]) * \
-                   self._dynasaur_definitions.get_unit_dimension_from_unit(json_object[JsonConstants.UNIT])
+                   self._dynasaur_definitions.get_unit_dimension_from_unit(unit=unit)
 
         elif JsonConstants.TYPE in json_object.keys():
             data_type = json_object[JsonConstants.TYPE]
